(function($)
{
  $.fn.jCloud = function(options)
	{
		var settings = $.extend({}, $.fn.jCloud.defaults, options)
		this.width(settings.width).height(settings.height).css('position', 'relative');
		
		if(settings.width == 'auto')
		{
			settings.width = this.width();
		}
		if(settings.height == 'auto')
		{
			settings.height = this.height();
		}
		
		var tree = makeChildNodes({
			'x':			0,
			'y':			0,
			'width':	settings.width,
			'height':	settings.height,
			'solid':	false
		});
		
		var elements = this.children().css('display', 'none').toArray();
		for(var key in elements)
		{
			if(!placeTag($(elements[key]), settings, tree))
			{
				break;
			}
		}
		
		return this;
  };
	
	$.fn.jCloud.defaults = {
		'width':		'auto',
		'height':		'auto',
		'vertical':	0.5,
		'step':			6
	};
	
	function placeTag(item, settings, tree)
	{
		var placed = false;
		if(Math.random() > settings.vertical)
		{
			var tag = {
				'x':			0,
				'y':			0,
				'width':	item.outerWidth(),
				'height':	item.outerHeight()
			};
			
			yOffset = Math.round(Math.random()*(settings.height - tag.height)/settings.step)*settings.step;
			Circle: for(var y = yOffset; y+tag.height < settings.height+yOffset; y += Math.floor(Math.random()*settings.step))
			{
				tag.y = y % (settings.height-tag.height);
				xOffset = Math.round(Math.random()*(settings.width - tag.width)/settings.step)*settings.step;
				for(var x = xOffset; x+tag.width < settings.width+xOffset; x += Math.floor(Math.random()*settings.step))
				{
					tag.x = x % (settings.width-tag.width);
					
					if(!intersectsWithTree(tag, tree))
					{
						item.css({
							'display':	'block',
							'position':	'absolute',
							'top':			tag.y,
							'left':			tag.x
						});
						
						tree = addToTree(tag, tree);
						
						placed = true;
						break Circle;
					}
				}
			}
		}
		else
		{
			var tag = {
				'x':			0,
				'y':			0,
				'width':	item.outerHeight(),
				'height':	item.outerWidth()
			};
			
			xOffset = Math.round(Math.random()*(settings.width - tag.width)/settings.step)*settings.step;
			Circle: for(var x = xOffset; x+tag.width < settings.width+xOffset; x += Math.floor(Math.random()*settings.step))
			{
				tag.x = x % (settings.width-tag.width)
				yOffset = Math.round(Math.random()*(settings.height - tag.height)/settings.step)*settings.step;
				for(var y = yOffset; y+tag.height < settings.height+yOffset; y += Math.floor(Math.random()*settings.step))
				{
					tag.y = y % (settings.height-tag.height);
					
					if(!intersectsWithTree(tag, tree))
					{
						item.addClass('vertical').css({
							'display':	'block',
							'position':	'absolute',
							'top':			tag.y + tag.height,
							'left':			tag.x
						});
						
						tree = addToTree(tag, tree);
						
						placed = true;
						break Circle;
					}
				}
			}
		}
		
		return placed;
	}
	
	function rectOverlap(A, B)
	{
		var xOverlap = valueInRange(A.x, B.x, B.x + B.width) || valueInRange(B.x, A.x, A.x + A.width);
		var yOverlap = valueInRange(A.y, B.y, B.y + B.height) || valueInRange(B.y, A.y, A.y + A.height);
		
		return xOverlap && yOverlap;
	}
	
	function valueInRange(value, min, max)
	{
		return (value <= max) && (value >= min);
	}
	
	function rectIntersection(A, B)
	{
		var intersection = {'solid': true};
		
		intersection.x = Math.max(A.x, B.x);
		intersection.y = Math.max(A.y, B.y);
		
		intersection.width = Math.min(A.x + A.width, B.x + B.width) - intersection.x;
		intersection.height = Math.min(A.y + A.height, B.y + B.height) - intersection.y;
		
		return intersection;
	}
	
	function makeChildNodes(node)
	{
		node.children = new Array();
		if(node.width > 100 || node.height > 100)
		{
			if(node.width > node.height)
			{
				node.children.push(makeChildNodes({
					'x':				node.x,
					'y':				node.y,
					'width':		node.width/2,
					'height':		node.height,
					'solid':		false}), makeChildNodes({
					'x':				node.x+node.width/2,
					'y':				node.y,
					'width':		node.width/2,
					'height':		node.height,
					'solid':		false}));
			}
			else
			{
				node.children.push(makeChildNodes({
					'x':				node.x,
					'y':				node.y,
					'width':		node.width,
					'height':		node.height/2,
					'solid':		false}), makeChildNodes({
					'x':				node.x,
					'y':				node.y+node.height/2,
					'width':		node.width,
					'height':		node.height/2,
					'solid':		false}));
			}
		}
		
		return node;
	}
	
	function intersectsWithTree(tag, tree)
	{
		var intersecting = false;
		
		if(rectOverlap(tree, tag))
		{
			if(tree.solid == true)
			{
				intersecting = true;
			}
			else
			{
				for(var childKey in tree.children)
				{
					if(intersectsWithTree(tag, tree.children[childKey]))
					{
						intersecting = true;
						break;
					}
				}
			}
		}
		
		return intersecting;
	}
	
	function addToTree(tag, tree)
	{
		if(tree.width > 100 || tree.height > 100)
		{
			for(var nodeKey in tree.children)
			{
				tree.children[nodeKey] = addToTree(tag, tree.children[nodeKey]);
			}
		}
		else
		{
			if(rectOverlap(tag, tree))
			{
				tree.children.push(rectIntersection(tag, tree));
			}
		}
		
		return tree;
	}
	
function var_dump(element, depth, tab)
{
	if(!depth)
		depth = 0;
	if(!tab)
		tab = '';

	tab += '\t';
	string = '';

	//Loop through all the child objects in element
	for(property in element)
	{
		//Add the name and value of the child object
		string += tab + property 			

		//Check if the child is an object
		if(typeof element[property] == 'object')
			string += '\n'+ var_dump(element[property], depth+1, tab) + '\n';
		else
			string += ': '+ element[property] + ''+ '\n';
	}

	//Ouput the result
	if(depth == 0) alert(string);	

	return string;
}
})(jQuery);